# Source Generated with Decompyle++
# File: in.pyo (Python 2.3)

__version__ = ' $Id: tables.py,v 1.72 2004/01/20 22:50:32 andy_robinson Exp $ '
__doc__ = "\nTables are created by passing the constructor a tuple of column widths, a tuple of row heights and the data in\nrow order. Drawing of the table can be controlled by using a TableStyle instance. This allows control of the\ncolor and weight of the lines (if any), and the font, alignment and padding of the text.\n\nNone values in the sequence of row heights or column widths, mean that the corresponding rows\nor columns should be automatically sized.\n\nAll the cell values should be convertible to strings; embedded newline '\\n' characters\ncause the value to wrap (ie are like a traditional linefeed).\n\nSee the test output from running this module as a script for a discussion of the method for constructing\ntables and table styles.\n"
from reportlab.platypus.flowables import Flowable, Image, Macro, PageBreak, Preformatted, Spacer, XBox, CondPageBreak, KeepTogether, TraceInfo, FailOnWrap, FailOnDraw
from reportlab.platypus.paragraph import Paragraph, cleanBlockQuotedText, ParaLines
from reportlab.platypus.paraparser import ParaFrag
from reportlab.platypus.frames import Frame
from reportlab.platypus.doctemplate import BaseDocTemplate, NextPageTemplate, PageTemplate, ActionFlowable, SimpleDocTemplate, FrameBreak, PageBegin
from reportlab.platypus.xpreformatted import XPreformatted
from reportlab import rl_config
from reportlab.lib.styles import PropertySet, getSampleStyleSheet, ParagraphStyle
from reportlab.lib import colors
from reportlab.lib.utils import fp_str
from reportlab.pdfbase import pdfmetrics
import operator
import string
from types import TupleType, ListType, StringType

class CellStyle(PropertySet):
    defaults = {
        'fontname': 'Times-Roman',
        'fontsize': 10,
        'leading': 12,
        'leftPadding': 6,
        'rightPadding': 6,
        'topPadding': 3,
        'bottomPadding': 3,
        'firstLineIndent': 0,
        'color': colors.black,
        'alignment': 'LEFT',
        'background': (1, 1, 1),
        'valign': 'BOTTOM' }

LINECAPS = {
    'butt': 0,
    'round': 1,
    'projecting': 2,
    'squared': 2 }
LINEJOINS = {
    'miter': 0,
    'round': 1,
    'bevel': 2 }

class CellStyle1(PropertySet):
    fontname = 'Times-Roman'
    fontsize = 10
    leading = 12
    leftPadding = 6
    rightPadding = 6
    topPadding = 3
    bottomPadding = 3
    firstLineIndent = 0
    color = colors.black
    alignment = 'LEFT'
    background = (1, 1, 1)
    valign = 'BOTTOM'
    
    def __init__(self, name, parent = None):
        self.name = name
        if parent is not None:
            parent.copy(self)
        

    
    def copy(self, result = None):
        if result is None:
            result = CellStyle1()
        
        for name in dir(self):
            setattr(result, name, gettattr(self, name))
        
        return result


CellStyle = CellStyle1

class TableStyle:
    
    def __init__(self, cmds = None, parent = None, **kw):
        commands = []
        if parent:
            commands = commands + parent.getCommands()
            self._opts = parent._opts
        
        if cmds:
            commands = commands + list(cmds)
        
        self._cmds = commands
        self._opts = { }
        self._opts.update(kw)

    
    def add(self, *cmd):
        self._cmds.append(cmd)

    
    def __repr__(self):
        L = map(repr, self._cmds)
        import string
        L = string.join(L, '  \n')
        return 'TableStyle(\n%s\n) # end TableStyle' % L

    
    def getCommands(self):
        return self._cmds


TableStyleType = type(TableStyle())
_SeqTypes = (TupleType, ListType)

def _rowLen(x):
    if not type(x) not in _SeqTypes and 1:
        pass
    return len(x)


class Table(Flowable):
    
    def __init__(self, data, colWidths = None, rowHeights = None, style = None, repeatRows = 0, repeatCols = 0, splitByRow = 1, emptyTableAction = None):
        global _emptyTableStyle
        self.hAlign = 'CENTER'
        self.vAlign = 'MIDDLE'
        if type(data) not in _SeqTypes:
            raise ValueError, '%s invalid data type' % self.identity()
        
        self._nrows = nrows = len(data)
        if nrows:
            self._ncols = ncols = max(map(_rowLen, data))
        elif colWidths:
            ncols = len(colWidths)
        else:
            ncols = 0
        if not emptyTableAction:
            emptyTableAction = rl_config.emptyTableAction
        
        if nrows:
            pass
        if not ncols:
            if emptyTableAction == 'error':
                raise ValueError, '%s must have at least a row and column' % self.identity()
            elif emptyTableAction == 'indicate':
                self.__class__ = Preformatted
                if '_emptyTableStyle' not in globals().keys():
                    _emptyTableStyle = ParagraphStyle('_emptyTableStyle')
                    _emptyTableStyle.textColor = colors.red
                    _emptyTableStyle.backColor = colors.yellow
                
                Preformatted.__init__(self, '%s(%d,%d)' % (self.__class__.__name__, nrows, ncols), _emptyTableStyle)
            elif emptyTableAction == 'ignore':
                self.__class__ = Spacer
                Spacer.__init__(self, 0, 0)
            else:
                raise ValueError, '%s bad emptyTableAction: "%s"' % (self.identity(), emptyTableAction)
            return None
        
        if colWidths is None:
            colWidths = ncols * [
                None]
        elif len(colWidths) != ncols:
            raise ValueError, '%s data error - %d columns in data but %d in grid' % (self.identity(), ncols, len(colWidths))
        
        if rowHeights is None:
            rowHeights = nrows * [
                None]
        elif len(rowHeights) != nrows:
            raise ValueError, '%s data error - %d rows in data but %d in grid' % (self.identity(), nrows, len(rowHeights))
        
        for i in range(nrows):
            if len(data[i]) != ncols:
                raise ValueError, '%s not enough data points in row %d!' % (self.identity(), i)
                continue
        
        self._cellvalues = data
        self._rowHeights = self._argH = rowHeights
        self._colWidths = self._argW = colWidths
        cellrows = []
        for i in range(nrows):
            cellcols = []
            for j in range(ncols):
                cellcols.append(CellStyle(`(i, j)`))
            
            cellrows.append(cellcols)
        
        self._cellStyles = cellrows
        self._bkgrndcmds = []
        self._linecmds = []
        self._spanCmds = []
        self.repeatRows = repeatRows
        self.repeatCols = repeatCols
        self.splitByRow = splitByRow
        if style:
            self.setStyle(style)
        

    
    def __repr__(self):
        '''incomplete, but better than nothing'''
        r = self._rowHeights
        c = self._colWidths
        cv = self._cellvalues
        import pprint
        import string
        cv = pprint.pformat(cv)
        cv = string.replace(cv, '\n', '\n  ')
        return '%s(\n rowHeights=%s,\n colWidths=%s,\n%s\n) # end table' % (self.__class__.__name__, r, c, cv)

    
    def identity(self, maxLen = 30):
        '''Identify our selves as well as possible'''
        vx = None
        nr = getattr(self, '_nrows', 'unknown')
        nc = getattr(self, '_ncols', 'unknown')
        cv = self._cellvalues
        if cv and 'unknown' not in (nr, nc):
            b = 0
            for i in xrange(nr):
                for j in xrange(nc):
                    v = cv[i][j]
                    t = type(v)
                    if t in _SeqTypes or isinstance(v, Flowable):
                        if not (t in _SeqTypes):
                            v = (v,)
                        
                        r = ''
                        for vij in v:
                            r = vij.identity(maxLen)
                            if r and r[-4:] != '>...':
                                break
                                continue
                        
                        if r and r[-4:] != '>...':
                            (ix, jx, vx, b) = (i, j, r, 1)
                        
                    elif not v is None and '':
                        pass
                    v = str(v)
                    (ix, jx, vx) = (i, j, v)
                    if not vx and t is StringType and 1:
                        pass
                    b = 0
                    if maxLen:
                        vx = vx[:maxLen]
                    
                    if b:
                        break
                        continue
                
                if b:
                    break
                    continue
            
        
        if vx:
            vx = ' with cell(%d,%d) containing\n%s' % (ix, jx, repr(vx))
        else:
            vx = '...'
        return '<%s at %d %d rows x %s cols>%s' % (self.__class__.__name__, id(self), nr, nc, vx)

    
    def _listCellGeom(self, V, w, s, W = None, H = None, aH = 72000):
        aW = w - s.leftPadding - s.rightPadding
        aH = aH - s.topPadding - s.bottomPadding
        t = 0
        w = 0
        canv = getattr(self, 'canv', None)
        for v in V:
            (vw, vh) = v.wrapOn(canv, aW, aH)
            if W is not None:
                W.append(vw)
            
            if H is not None:
                H.append(vh)
            
            w = max(w, vw)
            t = t + vh + v.getSpaceBefore() + v.getSpaceAfter()
        
        return (w, t - V[0].getSpaceBefore() - V[-1].getSpaceAfter())

    
    def _calc_width(self):
        W = self._argW
        canv = getattr(self, 'canv', None)
        saved = None
        if None in W:
            if self._spanCmds:
                colspans = self._colSpannedCells
            else:
                colspans = { }
            W = W[:]
            self._colWidths = W
            while None in W:
                j = W.index(None)
                
                f = lambda x, j = j: operator.getitem(x, j)
                V = map(f, self._cellvalues)
                S = map(f, self._cellStyles)
                w = 0
                i = 0
                for v, s in map(None, V, S):
                    if colspans.has_key((j, i)):
                        t = 0.0
                    else:
                        t = self._elementWidth(v, s)
                        if t is None:
                            raise ValueError, "Flowable %s in cell(%d,%d) can't have auto width\n%s" % (v.identity(30), i, j, self.identity(30))
                        
                        t = t + s.leftPadding + s.rightPadding
                    if t > w:
                        w = t
                    
                    i = i + 1
                
                W[j] = w
        
        width = 0
        self._colpositions = [
            0]
        for w in W:
            width = width + w
            self._colpositions.append(width)
        
        self._width = width

    
    def _elementWidth(self, v, s):
        t = type(v)
        if t in _SeqTypes:
            w = 0
            for e in v:
                ew = self._elementWidth(self, v)
                if ew is None:
                    return None
                
                w = max(w, ew)
            
            return w
        elif isinstance(v, Flowable) and v._fixedWidth:
            return v.width
        elif t is not StringType:
            if not v is None and '':
                pass
            v = str(v)
        
        v = string.split(v, '\n')
        return max(map((lambda a, b = s.fontname, c = s.fontsize, d = pdfmetrics.stringWidth: d(a, b, c)), v))

    
    def _calc_height(self, availHeight):
        H = self._argH
        W = self._argW
        canv = getattr(self, 'canv', None)
        saved = None
        if self._spanCmds:
            spans = self._rowSpannedCells
        else:
            spans = { }
        hmax = lim = len(H)
        longTable = getattr(self, '_longTableOptimize', None)
        if None in H:
            if canv:
                saved = (canv._fontname, canv._fontsize, canv._leading)
            
            H = H[:]
            self._rowHeights = H
            while None in H:
                i = H.index(None)
                if longTable:
                    hmax = i
                    height = reduce(operator.add, H[:i], 0)
                    if height > availHeight:
                        break
                    
                
                V = self._cellvalues[i]
                S = self._cellStyles[i]
                h = 0
                j = 0
                for v, s, w in map(None, V, S, W):
                    if spans.has_key((j, i)):
                        t = 0.0
                    else:
                        t = type(v)
                        if t in _SeqTypes or isinstance(v, Flowable):
                            if not (t in _SeqTypes):
                                v = (v,)
                            
                            if w is None:
                                raise ValueError, "Flowable %s in cell(%d,%d) can't have auto width in\n%s" % (v[0].identity(30), i, j, self.identity(30))
                            
                            if canv:
                                if not s.leading:
                                    pass
                                (canv._fontname, canv._fontsize, canv._leading) = (s.fontname, s.fontsize, 1.2 * s.fontsize)
                            
                            (dW, t) = self._listCellGeom(v, w, s)
                            if canv:
                                (canv._fontname, canv._fontsize, canv._leading) = saved
                            
                            dW = dW + s.leftPadding + s.rightPadding
                            if not (rl_config.allowTableBoundsErrors) and dW > w:
                                raise 'LayoutError', 'Flowable %s (%sx%s points) too wide for cell(%d,%d) (%sx* points) in\n%s' % (v[0].identity(30), fp_str(dW), fp_str(t), i, j, fp_str(w), self.identity(30))
                            
                        elif t is not StringType:
                            if not v is None and '':
                                pass
                            v = str(v)
                        
                        v = string.split(v, '\n')
                        t = s.leading * len(v)
                        t = t + s.bottomPadding + s.topPadding
                    if t > h:
                        h = t
                    
                    j = j + 1
                
                H[i] = h
            if None not in H:
                hmax = lim
            
        
        height = self._height = reduce(operator.add, H[:hmax], 0)
        self._rowpositions = [
            height]
        for h in H[:hmax]:
            height = height - h
            self._rowpositions.append(height)
        
        self._hmax = hmax

    
    def _calc(self, availWidth, availHeight):
        if None in self._colWidths:
            if self._hasVariWidthElements():
                self._calcPreliminaryWidths(availWidth)
            
        
        if self._spanCmds:
            self._calcSpanRanges()
        
        self._calc_height(availHeight)
        if hasattr(self, '_width_calculated_once'):
            return None
        
        self._width_calculated_once = 1
        self._calc_width()
        if self._spanCmds:
            self._calcSpanRects()
        

    
    def _hasVariWidthElements(self, upToRow = None):
        '''Check for flowables in table cells and warn up front.

        Allow a couple which we know are fixed size such as
        images and graphics.'''
        bad = 0
        if upToRow is None:
            upToRow = self._nrows
        
        for row in range(min(self._nrows, upToRow)):
            for col in range(self._ncols):
                value = self._cellvalues[row][col]
                if not self._canGetWidth(value):
                    bad = 1
                    continue
            
        
        return bad

    
    def _canGetWidth(self, thing):
        '''Can we work out the width quickly?'''
        if type(thing) in (ListType, TupleType):
            for elem in thing:
                if not self._canGetWidth(elem):
                    return 0
                    continue
            
            return 1
        elif isinstance(thing, Flowable):
            return thing._fixedWidth
        else:
            return 1

    
    def _calcPreliminaryWidths(self, availWidth):
        '''Fallback algorithm for when main one fails.

        Where exact width info not given but things like
        paragraphs might be present, do a preliminary scan
        and assign some sensible values - just divide up
        all unsizeable columns by the remaining space.'''
        verbose = 0
        totalDefined = 0.0
        numberUndefined = 0
        for w in self._colWidths:
            if w is None:
                numberUndefined = numberUndefined + 1
                continue
            totalDefined = totalDefined + w
        
        if verbose:
            print 'prelim width calculation.  %d columns, %d undefined width, %0.2f units remain' % (self._ncols, numberUndefined, availWidth - totalDefined)
        
        given = []
        sizeable = []
        unsizeable = []
        for colNo in range(self._ncols):
            if self._colWidths[colNo] is None:
                siz = 1
                for rowNo in range(self._nrows):
                    value = self._cellvalues[rowNo][colNo]
                    if not self._canGetWidth(value):
                        siz = 0
                        break
                        continue
                
                if siz:
                    sizeable.append(colNo)
                else:
                    unsizeable.append(colNo)
            siz
            given.append(colNo)
        
        if len(given) == self._ncols:
            return None
        
        if verbose:
            print 'predefined width:   ', given
        
        if verbose:
            print 'uncomputable width: ', unsizeable
        
        if verbose:
            print 'computable width:    ', sizeable
        
        newColWidths = list(self._colWidths)
        guessColWidth = (availWidth - totalDefined) / (len(unsizeable) + len(sizeable))
        if verbose:
            print 'assigning width %0.2f to all undefined columns' % guessColWidth
        
        for colNo in sizeable:
            newColWidths[colNo] = guessColWidth
        
        for colNo in unsizeable:
            newColWidths[colNo] = guessColWidth
        
        self._colWidths = newColWidths
        self._argW = newColWidths
        if verbose:
            print 'new widths are:', self._colWidths
        

    
    def _calcSpanRanges(self):
        '''Work out rects for tables which do row and column spanning.

        This creates some mappings to let the later code determine
        if a cell is part of a "spanned" range.
        self._spanRanges shows the \'coords\' in integers of each
        \'cell range\', or None if it was clobbered:
          (col, row) -> (col0, row0, col1, row1)

        Any cell not in the key is not part of a spanned region
        '''
        spanRanges = { }
        for row in range(self._nrows):
            for col in range(self._ncols):
                spanRanges[(col, row)] = (col, row, col, row)
            
        
        for cmd, start, stop in self._spanCmds:
            (x0, y0) = start
            (x1, y1) = stop
            if x0 < 0:
                x0 = x0 + self._ncols
            
            if x1 < 0:
                x1 = x1 + self._ncols
            
            if y0 < 0:
                y0 = y0 + self._nrows
            
            if y1 < 0:
                y1 = y1 + self._nrows
            
            if x0 > x1:
                (x0, x1) = (x1, x0)
            
            if y0 > y1:
                (y0, y1) = (y1, y0)
            
            for y in range(y0, y1 + 1):
                for x in range(x0, x1 + 1):
                    spanRanges[(x, y)] = None
                
            
            spanRanges[(x0, y0)] = (x0, y0, x1, y1)
        
        self._spanRanges = spanRanges
        colSpannedCells = { }
        for key, value in spanRanges.items():
            if value is None:
                colSpannedCells[key] = 1
                continue
            if len(value) == 4:
                if value[0] == value[2]:
                    pass
                else:
                    colSpannedCells[key] = 1
            value[0] == value[2]
        
        self._colSpannedCells = colSpannedCells
        rowSpannedCells = { }
        for key, value in spanRanges.items():
            if value is None:
                rowSpannedCells[key] = 1
                continue
            if len(value) == 4:
                if value[1] == value[3]:
                    pass
                else:
                    rowSpannedCells[key] = 1
            value[1] == value[3]
        
        self._rowSpannedCells = rowSpannedCells

    
    def _calcSpanRects(self):
        """Work out rects for tables which do row and column spanning.

        Based on self._spanRanges, which is already known,
        and the widths which were given or previously calculated,
        self._spanRects shows the real coords for drawing:
          (col, row) -> (x, y, width, height)

        for each cell.  Any cell which 'does not exist' as another
        has spanned over it will get a None entry on the right
        """
        spanRects = { }
        for coord, value in self._spanRanges.items():
            if value is None:
                spanRects[coord] = None
                continue
            (col, row) = coord
            (col0, row0, col1, row1) = value
            x = self._colpositions[col0]
            y = self._rowpositions[row1 + 1]
            width = self._colpositions[col1 + 1] - x
            height = self._rowpositions[row0] - y
            spanRects[coord] = (x, y, width, height)
        
        self._spanRects = spanRects

    
    def setStyle(self, tblstyle):
        if type(tblstyle) is not TableStyleType:
            tblstyle = TableStyle(tblstyle)
        
        for cmd in tblstyle.getCommands():
            self._addCommand(cmd)
        
        for k, v in tblstyle._opts.items():
            setattr(self, k, v)
        

    
    def _addCommand(self, cmd):
        if cmd[0] == 'BACKGROUND':
            self._bkgrndcmds.append(cmd)
        elif cmd[0] == 'SPAN':
            self._spanCmds.append(cmd)
        elif _isLineCommand(cmd):
            cmd = tuple(cmd)
            if len(cmd) < 5:
                raise ValueError('bad line command ' + str(cmd))
            
            if len(cmd) < 6:
                cmd = cmd + (1,)
            else:
                cap = cmd[5]
                
                try:
                    if type(cap) is not type(int):
                        cap = LINECAPS[cap]
                    elif cap < 0 or cap > 2:
                        raise ValueError
                    
                    cmd = cmd[:5] + (cap,) + cmd[6:]
                except:
                    ValueError('Bad cap value %s in %s' % (cap, str(cmd)))

            if len(cmd) < 7:
                cmd = cmd + (None,)
            
            if len(cmd) < 8:
                cmd = cmd + (1,)
            else:
                join = cmd[7]
                
                try:
                    if type(join) is not type(int):
                        join = LINEJOINS[cap]
                    elif join < 0 or join > 2:
                        raise ValueError
                    
                    cmd = cmd[:7] + (join,)
                except:
                    ValueError('Bad join value %s in %s' % (join, str(cmd)))

            self._linecmds.append(cmd)
        else:
            (sc, sr) = (op,)
            (ec, er) = ()
            values = (cmd[:3], cmd[3:])
            if sc < 0:
                sc = sc + self._ncols
            
            if ec < 0:
                ec = ec + self._ncols
            
            if sr < 0:
                sr = sr + self._nrows
            
            if er < 0:
                er = er + self._nrows
            
            for i in range(sr, er + 1):
                for j in range(sc, ec + 1):
                    _setCellStyle(self._cellStyles, i, j, op, values)
                
            

    
    def _drawLines(self):
        (ccap, cdash, cjoin) = (None, None, None)
        self.canv.saveState()
        for sc, sr in self._linecmds:
            (ec, er) = None
            weight = None
            color = None
            cap = None
            dash = None
            join = None
            if type(sr) is type('') and sr.startswith('split'):
                continue
            
            if sc < 0:
                sc = sc + self._ncols
            
            if ec < 0:
                ec = ec + self._ncols
            
            if sr < 0:
                sr = sr + self._nrows
            
            if er < 0:
                er = er + self._nrows
            
            if ccap != cap:
                self.canv.setLineCap(cap)
                ccap = cap
            
            getattr(self, _LineOpMap.get(op, '_drawUnknown'))((sc, sr), (ec, er), weight, color)
        
        self.canv.restoreState()
        self._curcolor = None

    
    def _drawUnknown(self, .2, .4, weight, color):
        (sc, sr) = .2
        (ec, er) = .4
        raise ValueError, "Unknown line command '%s'" % op

    
    def _drawGrid(self, .2, .4, weight, color):
        (sc, sr) = .2
        (ec, er) = .4
        self._drawBox((sc, sr), (ec, er), weight, color)
        self._drawInnerGrid((sc, sr), (ec, er), weight, color)

    
    def _drawBox(self, .2, .4, weight, color):
        (sc, sr) = .2
        (ec, er) = .4
        self._drawHLines((sc, sr), (ec, sr), weight, color)
        self._drawHLines((sc, er + 1), (ec, er + 1), weight, color)
        self._drawVLines((sc, sr), (sc, er), weight, color)
        self._drawVLines((ec + 1, sr), (ec + 1, er), weight, color)

    
    def _drawInnerGrid(self, .2, .4, weight, color):
        (sc, sr) = .2
        (ec, er) = .4
        self._drawHLines((sc, sr + 1), (ec, er), weight, color)
        self._drawVLines((sc + 1, sr), (ec, er), weight, color)

    
    def _prepLine(self, weight, color):
        if color != self._curcolor:
            self.canv.setStrokeColor(color)
            self._curcolor = color
        
        if weight != self._curweight:
            self.canv.setLineWidth(weight)
            self._curweight = weight
        

    
    def _drawHLines(self, .2, .4, weight, color):
        (sc, sr) = .2
        (ec, er) = .4
        ecp = self._colpositions[sc:ec + 2]
        rp = self._rowpositions[sr:er + 1]
        if len(ecp) <= 1 or len(rp) < 1:
            return None
        
        self._prepLine(weight, color)
        scp = ecp[0]
        ecp = ecp[-1]
        for rowpos in rp:
            self.canv.line(scp, rowpos, ecp, rowpos)
        

    
    def _drawHLinesB(self, .2, .4, weight, color):
        (sc, sr) = .2
        (ec, er) = .4
        self._drawHLines((sc, sr + 1), (ec, er + 1), weight, color)

    
    def _drawVLines(self, .2, .4, weight, color):
        (sc, sr) = .2
        (ec, er) = .4
        erp = self._rowpositions[sr:er + 2]
        cp = self._colpositions[sc:ec + 1]
        if len(erp) <= 1 or len(cp) < 1:
            return None
        
        self._prepLine(weight, color)
        srp = erp[0]
        erp = erp[-1]
        for colpos in cp:
            self.canv.line(colpos, srp, colpos, erp)
        

    
    def _drawVLinesA(self, .2, .4, weight, color):
        (sc, sr) = .2
        (ec, er) = .4
        self._drawVLines((sc + 1, sr), (ec + 1, er), weight, color)

    
    def wrap(self, availWidth, availHeight):
        self._calc(availWidth, availHeight)
        self.availWidth = availWidth
        return (self._width, self._height)

    
    def onSplit(self, T, byRow = 1):
        '''
        This method will be called when the Table is split.
        Special purpose tables can override to do special stuff.
        '''
        pass

    
    def _cr_0(self, n, cmds):
        for c in cmds:
            c = tuple(c)
            (sc, sr) = ()
            (ec, er) = c[1:3]
            if sr >= n:
                continue
            
            if er >= n:
                er = n - 1
            
            self._addCommand((c[0],) + ((sc, sr), (ec, er)) + c[3:])
        

    
    def _cr_1_1(self, n, repeatRows, cmds):
        for c in cmds:
            c = tuple(c)
            (sc, sr) = ()
            (ec, er) = c[1:3]
            if sr in ('splitfirst', 'splitlast'):
                self._addCommand(c)
                continue
            if sr >= 0 and sr >= repeatRows and sr < n and er >= 0 and er < n:
                continue
            
            if sr >= repeatRows and sr < n:
                sr = repeatRows
            elif sr >= repeatRows and sr >= n:
                sr = sr + repeatRows - n
            
            if er >= repeatRows and er < n:
                er = repeatRows
            elif er >= repeatRows and er >= n:
                er = er + repeatRows - n
            
            self._addCommand((c[0],) + ((sc, sr), (ec, er)) + c[3:])
        

    
    def _cr_1_0(self, n, cmds):
        for c in cmds:
            c = tuple(c)
            (sc, sr) = ()
            (ec, er) = c[1:3]
            if sr in ('splitfirst', 'splitlast'):
                self._addCommand(c)
                continue
            if er >= 0 and er < n:
                continue
            
            if sr >= 0 and sr < n:
                sr = 0
            
            if sr >= n:
                sr = sr - n
            
            if er >= n:
                er = er - n
            
            self._addCommand((c[0],) + ((sc, sr), (ec, er)) + c[3:])
        

    
    def _splitRows(self, availHeight):
        h = 0
        n = 0
        lim = len(self._rowHeights)
        while n < self._hmax:
            hn = h + self._rowHeights[n]
            if hn > availHeight:
                break
            
            h = hn
            n = n + 1
        if n <= self.repeatRows:
            return []
        
        if n == lim:
            return [
                self]
        
        repeatRows = self.repeatRows
        repeatCols = self.repeatCols
        splitByRow = self.splitByRow
        data = self._cellvalues
        R0 = self.__class__(data[:n], self._colWidths, self._argH[:n], repeatRows = repeatRows, repeatCols = repeatCols, splitByRow = splitByRow)
        R0._cellStyles = self._cellStyles[:n]
        A = []
        for sc, sr in self._linecmds:
            (ec, er) = None
            weight = None
            color = None
            cap = None
            dash = None
            join = None
            if type(sr) is type('') and sr.startswith('split'):
                A.append((op, (sc, sr), (ec, sr), weight, color, cap, dash, join))
                if sr == 'splitlast':
                    sr = er = n - 1
                elif sr == 'splitfirst':
                    sr = n
                    er = n
                
            
            if sc < 0:
                sc = sc + self._ncols
            
            if ec < 0:
                ec = ec + self._ncols
            
            if sr < 0:
                sr = sr + self._nrows
            
            if er < 0:
                er = er + self._nrows
            
            if op in ('BOX', 'OUTLINE', 'GRID'):
                if sr < n and er >= n:
                    A.append(('LINEABOVE', (sc, sr), (ec, sr), weight, color, cap, dash, join))
                    A.append(('LINEBEFORE', (sc, sr), (sc, er), weight, color, cap, dash, join))
                    A.append(('LINEAFTER', (ec, sr), (ec, er), weight, color, cap, dash, join))
                    A.append(('LINEBELOW', (sc, er), (ec, er), weight, color, cap, dash, join))
                    if op == 'GRID':
                        A.append(('LINEBELOW', (sc, n - 1), (ec, n - 1), weight, color, cap, dash, join))
                        A.append(('LINEABOVE', (sc, n), (ec, n), weight, color, cap, dash, join))
                        A.append(('INNERGRID', (sc, sr), (ec, er), weight, color, cap, dash, join))
                    
                else:
                    A.append((op, (sc, sr), (ec, er), weight, color, cap, dash, join))
            er >= n
            if op in ('INNERGRID', 'LINEABOVE'):
                if sr < n and er >= n:
                    A.append(('LINEBELOW', (sc, n - 1), (ec, n - 1), weight, color, cap, dash, join))
                    A.append(('LINEABOVE', (sc, n), (ec, n), weight, color, cap, dash, join))
                
                A.append((op, (sc, sr), (ec, er), weight, color, cap, dash, join))
                continue
            if op == 'LINEBELOW':
                if sr < n and er >= n - 1:
                    A.append(('LINEABOVE', (sc, n), (ec, n), weight, color, cap, dash, join))
                
                A.append((op, (sc, sr), (ec, er), weight, color))
                continue
            if op == 'LINEABOVE':
                if sr <= n and er >= n:
                    A.append(('LINEBELOW', (sc, n - 1), (ec, n - 1), weight, color, cap, dash, join))
                
                A.append((op, (sc, sr), (ec, er), weight, color, cap, dash, join))
                continue
            A.append((op, (sc, sr), (ec, er), weight, color, cap, dash, join))
        
        R0._cr_0(n, A)
        R0._cr_0(n, self._bkgrndcmds)
        if repeatRows:
            R1 = self.__class__(data[:repeatRows] + data[n:], self._colWidths, self._argH[:repeatRows] + self._argH[n:], repeatRows = repeatRows, repeatCols = repeatCols, splitByRow = splitByRow)
            R1._cellStyles = self._cellStyles[:repeatRows] + self._cellStyles[n:]
            R1._cr_1_1(n, repeatRows, A)
            R1._cr_1_1(n, repeatRows, self._bkgrndcmds)
        else:
            R1 = self.__class__(data[n:], self._colWidths, self._argH[n:], repeatRows = repeatRows, repeatCols = repeatCols, splitByRow = splitByRow)
            R1._cellStyles = self._cellStyles[n:]
            R1._cr_1_0(n, A)
            R1._cr_1_0(n, self._bkgrndcmds)
        R0.hAlign = R1.hAlign = self.hAlign
        R0.vAlign = R1.vAlign = self.vAlign
        self.onSplit(R0)
        self.onSplit(R1)
        return [
            R0,
            R1]

    
    def split(self, availWidth, availHeight):
        self._calc(availWidth, availHeight)
        if self.splitByRow:
            if not (rl_config.allowTableBoundsErrors) and self._width > availWidth:
                return []
            
            return self._splitRows(availHeight)
        else:
            raise NotImplementedError

    
    def draw(self):
        self._curweight = self._curcolor = self._curcellstyle = None
        self._drawBkgrnd()
        self._drawLines()
        if self._spanCmds == []:
            for row, rowstyle, rowpos, rowheight in map(None, self._cellvalues, self._cellStyles, self._rowpositions[1:], self._rowHeights):
                for cellval, cellstyle, colpos, colwidth in map(None, row, rowstyle, self._colpositions[:-1], self._colWidths):
                    self._drawCell(cellval, cellstyle, (colpos, rowpos), (colwidth, rowheight))
                
            
        else:
            for rowNo in range(self._nrows):
                for colNo in range(self._ncols):
                    cellRect = self._spanRects[(colNo, rowNo)]
                    if cellRect is not None:
                        (x, y, width, height) = cellRect
                        cellval = self._cellvalues[rowNo][colNo]
                        cellstyle = self._cellStyles[rowNo][colNo]
                        self._drawCell(cellval, cellstyle, (x, y), (width, height))
                        continue
                
            

    
    def _drawBkgrnd(self):
        nrows = self._nrows
        ncols = self._ncols
        for sc, sr in self._bkgrndcmds:
            (ec, er) = None
            arg = None
            if sc < 0:
                sc = sc + ncols
            
            if ec < 0:
                ec = ec + ncols
            
            if sr < 0:
                sr = sr + nrows
            
            if er < 0:
                er = er + nrows
            
            x0 = self._colpositions[sc]
            y0 = self._rowpositions[sr]
            x1 = self._colpositions[min(ec + 1, ncols)]
            y1 = self._rowpositions[min(er + 1, nrows)]
            (w, h) = (x1 - x0, y1 - y0)
            canv = self.canv
            if callable(arg):
                apply(arg, (self, canv, x0, y0, w, h))
                continue
            canv.setFillColor(colors.toColor(arg))
            canv.rect(x0, y0, w, h, stroke = 0, fill = 1)
        

    
    def _drawCell(self, cellval, cellstyle, .6, .8):
        (colpos, rowpos) = .6
        (colwidth, rowheight) = .8
        if self._curcellstyle is not cellstyle:
            cur = self._curcellstyle
            if cur is None or cellstyle.color != cur.color:
                self.canv.setFillColor(cellstyle.color)
            
            if cur is None and cellstyle.leading != cur.leading and cellstyle.fontname != cur.fontname or cellstyle.fontsize != cur.fontsize:
                self.canv.setFont(cellstyle.fontname, cellstyle.fontsize, cellstyle.leading)
            
            self._curcellstyle = cellstyle
        
        just = cellstyle.alignment
        valign = cellstyle.valign
        n = type(cellval)
        if n in _SeqTypes or isinstance(cellval, Flowable):
            if not (n in _SeqTypes):
                cellval = (cellval,)
            
            W = []
            H = []
            (w, h) = self._listCellGeom(cellval, colwidth, cellstyle, W = W, H = H, aH = rowheight)
            if valign == 'TOP':
                y = rowpos + rowheight - cellstyle.topPadding
            elif valign == 'BOTTOM':
                y = rowpos + cellstyle.bottomPadding + h
            else:
                y = rowpos + ((rowheight + cellstyle.bottomPadding - cellstyle.topPadding) + h) / 2.0
            y = y + cellval[0].getSpaceBefore()
            for v, w, h in map(None, cellval, W, H):
                if just == 'LEFT':
                    x = colpos + cellstyle.leftPadding
                elif just == 'RIGHT':
                    x = colpos + colwidth - cellstyle.rightPadding - w
                elif just in ('CENTRE', 'CENTER'):
                    x = colpos + (colwidth + cellstyle.leftPadding - cellstyle.rightPadding - w) / 2.0
                else:
                    raise ValueError, 'Invalid justification %s' % just
                y = y - v.getSpaceBefore()
                y = y - h
                v.drawOn(self.canv, x, y)
                y = y - v.getSpaceAfter()
            
        elif just == 'LEFT':
            draw = self.canv.drawString
            x = colpos + cellstyle.leftPadding
        elif just in ('CENTRE', 'CENTER'):
            draw = self.canv.drawCentredString
            x = colpos + colwidth * 0.5
        elif just == 'RIGHT':
            draw = self.canv.drawRightString
            x = colpos + colwidth - cellstyle.rightPadding
        else:
            raise ValueError, 'Invalid justification %s' % just
        if n is StringType:
            val = cellval
        else:
            val = str(cellval)
        vals = string.split(val, '\n')
        n = len(vals)
        leading = cellstyle.leading
        fontsize = cellstyle.fontsize
        if valign == 'BOTTOM':
            y = rowpos + cellstyle.bottomPadding + n * leading - fontsize
        elif valign == 'TOP':
            y = rowpos + rowheight - cellstyle.topPadding - fontsize
        elif valign == 'MIDDLE':
            y = rowpos + ((cellstyle.bottomPadding + rowheight - cellstyle.topPadding) + (n - 1) * leading) / 2.0
        else:
            raise ValueError, "Bad valign: '%s'" % str(valign)
        for v in vals:
            draw(x, y, v)
            y = y - leading
        


_LineOpMap = {
    'GRID': '_drawGrid',
    'BOX': '_drawBox',
    'OUTLINE': '_drawBox',
    'INNERGRID': '_drawInnerGrid',
    'LINEBELOW': '_drawHLinesB',
    'LINEABOVE': '_drawHLines',
    'LINEBEFORE': '_drawVLines',
    'LINEAFTER': '_drawVLinesA' }

class LongTable(Table):
    """Henning von Bargen's changes will be active"""
    _longTableOptimize = 1

LINECOMMANDS = _LineOpMap.keys()

def _isLineCommand(cmd):
    return cmd[0] in LINECOMMANDS


def _setCellStyle(cellStyles, i, j, op, values):
    new = cellStyles[i][j]
    if op == 'FONT':
        n = len(values)
        new.fontname = values[0]
        if n > 1:
            new.fontsize = values[1]
            if n > 2:
                new.leading = values[2]
            else:
                new.leading = new.fontsize * 1.2
        
    elif op in ('FONTNAME', 'FACE'):
        new.fontname = values[0]
    elif op in ('SIZE', 'FONTSIZE'):
        new.fontsize = values[0]
    elif op == 'LEADING':
        new.leading = values[0]
    elif op == 'TEXTCOLOR':
        new.color = colors.toColor(values[0], colors.Color(0, 0, 0))
    elif op in ('ALIGN', 'ALIGNMENT'):
        new.alignment = values[0]
    elif op == 'VALIGN':
        new.valign = values[0]
    elif op == 'LEFTPADDING':
        new.leftPadding = values[0]
    elif op == 'RIGHTPADDING':
        new.rightPadding = values[0]
    elif op == 'TOPPADDING':
        new.topPadding = values[0]
    elif op == 'BOTTOMPADDING':
        new.bottomPadding = values[0]
    

GRID_STYLE = TableStyle([
    ('GRID', (0, 0), (-1, -1), 0.25, colors.black),
    ('ALIGN', (1, 1), (-1, -1), 'RIGHT')])
BOX_STYLE = TableStyle([
    ('BOX', (0, 0), (-1, -1), 0.5, colors.black),
    ('ALIGN', (1, 1), (-1, -1), 'RIGHT')])
LABELED_GRID_STYLE = TableStyle([
    ('INNERGRID', (0, 0), (-1, -1), 0.25, colors.black),
    ('BOX', (0, 0), (-1, -1), 2, colors.black),
    ('LINEBELOW', (0, 0), (-1, 0), 2, colors.black),
    ('LINEAFTER', (0, 0), (0, -1), 2, colors.black),
    ('ALIGN', (1, 1), (-1, -1), 'RIGHT')])
COLORED_GRID_STYLE = TableStyle([
    ('INNERGRID', (0, 0), (-1, -1), 0.25, colors.black),
    ('BOX', (0, 0), (-1, -1), 2, colors.red),
    ('LINEBELOW', (0, 0), (-1, 0), 2, colors.black),
    ('LINEAFTER', (0, 0), (0, -1), 2, colors.black),
    ('ALIGN', (1, 1), (-1, -1), 'RIGHT')])
LIST_STYLE = TableStyle([
    ('LINEABOVE', (0, 0), (-1, 0), 2, colors.green),
    ('LINEABOVE', (0, 1), (-1, -1), 0.25, colors.black),
    ('LINEBELOW', (0, -1), (-1, -1), 2, colors.green),
    ('ALIGN', (1, 1), (-1, -1), 'RIGHT')])

def test():
    inch = inch
    import reportlab.lib.units
    rowheights = (24, 16, 16, 16, 16)
    rowheights2 = (24, 16, 16, 16, 30)
    colwidths = (50, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32)
    data = (('', 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'), ('Mugs', 0, 4, 17, 3, 21, 47, 12, 33, 2, -2, 44, 89), ('T-Shirts', 0, 42, 9, -3, 16, 4, 72, 89, 3, 19, 32, 119), ('Key Ring', 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 2, 13), ('Hats', 893, 912, '1,212', 643, 789, 159, 888, '1,298', 832, 453, '1,344', '2,843'))
    data2 = (('', 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'), ('Mugs', 0, 4, 17, 3, 21, 47, 12, 33, 2, -2, 44, 89), ('T-Shirts', 0, 42, 9, -3, 16, 4, 72, 89, 3, 19, 32, 119), ('Key Ring', 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 2, 13), ('Hats\nLarge', 893, 912, '1,212', 643, 789, 159, 888, '1,298', 832, 453, '1,344', '2,843'))
    styleSheet = getSampleStyleSheet()
    lst = []
    lst.append(Paragraph('Tables', styleSheet['Heading1']))
    lst.append(Paragraph(__doc__, styleSheet['BodyText']))
    lst.append(Paragraph('The Tables (shown in different styles below) were created using the following code:', styleSheet['BodyText']))
    lst.append(Preformatted("\n    colwidths = (50, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32)\n    rowheights = (24, 16, 16, 16, 16)\n    data = (\n        ('', 'Jan', 'Feb', 'Mar','Apr','May', 'Jun',\n           'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'),\n        ('Mugs', 0, 4, 17, 3, 21, 47, 12, 33, 2, -2, 44, 89),\n        ('T-Shirts', 0, 42, 9, -3, 16, 4, 72, 89, 3, 19, 32, 119),\n        ('Key Ring', 0,0,0,0,0,0,1,0,0,0,2,13),\n        ('Hats', 893, 912, '1,212', 643, 789, 159,\n             888, '1,298', 832, 453, '1,344','2,843')\n        )\n    t = Table(data, colwidths, rowheights)\n    ", styleSheet['Code'], dedent = 4))
    lst.append(Paragraph('\n    You can then give the Table a TableStyle object to control its format. The first TableStyle used was\n    created as follows:\n    ', styleSheet['BodyText']))
    lst.append(Preformatted("\nGRID_STYLE = TableStyle(\n    [('GRID', (0,0), (-1,-1), 0.25, colors.black),\n     ('ALIGN', (1,1), (-1,-1), 'RIGHT')]\n    )\n    ", styleSheet['Code']))
    lst.append(Paragraph('\n    TableStyles are created by passing in a list of commands. There are two types of commands - line commands\n    and cell formatting commands. In all cases, the first three elements of a command are the command name,\n    the starting cell and the ending cell.\n    ', styleSheet['BodyText']))
    lst.append(Paragraph('\n    Line commands always follow this with the weight and color of the desired lines. Colors can be names,\n    or they can be specified as a (R,G,B) tuple, where R, G and B are floats and (0,0,0) is black. The line\n    command names are: GRID, BOX, OUTLINE, INNERGRID, LINEBELOW, LINEABOVE, LINEBEFORE\n    and LINEAFTER. BOX and OUTLINE are equivalent, and GRID is the equivalent of applying both BOX and\n    INNERGRID.\n    ', styleSheet['BodyText']))
    lst.append(Paragraph('\n    Cell formatting commands are:\n    ', styleSheet['BodyText']))
    lst.append(Paragraph('\n    FONT - takes fontname, fontsize and (optional) leading.\n    ', styleSheet['Definition']))
    lst.append(Paragraph('\n    TEXTCOLOR - takes a color name or (R,G,B) tuple.\n    ', styleSheet['Definition']))
    lst.append(Paragraph('\n    ALIGNMENT (or ALIGN) - takes one of LEFT, RIGHT and CENTRE (or CENTER).\n    ', styleSheet['Definition']))
    lst.append(Paragraph('\n    LEFTPADDING - defaults to 6.\n    ', styleSheet['Definition']))
    lst.append(Paragraph('\n    RIGHTPADDING - defaults to 6.\n    ', styleSheet['Definition']))
    lst.append(Paragraph('\n    BOTTOMPADDING - defaults to 3.\n    ', styleSheet['Definition']))
    lst.append(Paragraph('\n    A tablestyle is applied to a table by calling Table.setStyle(tablestyle).\n    ', styleSheet['BodyText']))
    t = Table(data, colwidths, rowheights)
    t.setStyle(GRID_STYLE)
    lst.append(PageBreak())
    lst.append(Paragraph('This is GRID_STYLE\n', styleSheet['BodyText']))
    lst.append(t)
    t = Table(data, colwidths, rowheights)
    t.setStyle(BOX_STYLE)
    lst.append(Paragraph('This is BOX_STYLE\n', styleSheet['BodyText']))
    lst.append(t)
    lst.append(Paragraph('\n    It was created as follows:\n    ', styleSheet['BodyText']))
    lst.append(Preformatted("\nBOX_STYLE = TableStyle(\n    [('BOX', (0,0), (-1,-1), 0.50, colors.black),\n     ('ALIGN', (1,1), (-1,-1), 'RIGHT')]\n    )\n    ", styleSheet['Code']))
    t = Table(data, colwidths, rowheights)
    t.setStyle(LABELED_GRID_STYLE)
    lst.append(Paragraph('This is LABELED_GRID_STYLE\n', styleSheet['BodyText']))
    lst.append(t)
    t = Table(data2, colwidths, rowheights2)
    t.setStyle(LABELED_GRID_STYLE)
    lst.append(Paragraph('This is LABELED_GRID_STYLE ILLUSTRATES EXPLICIT LINE SPLITTING WITH NEWLINE (different heights and data)\n', styleSheet['BodyText']))
    lst.append(t)
    lst.append(Paragraph('\n    It was created as follows:\n    ', styleSheet['BodyText']))
    lst.append(Preformatted("\nLABELED_GRID_STYLE = TableStyle(\n    [('INNERGRID', (0,0), (-1,-1), 0.25, colors.black),\n     ('BOX', (0,0), (-1,-1), 2, colors.black),\n     ('LINEBELOW', (0,0), (-1,0), 2, colors.black),\n     ('LINEAFTER', (0,0), (0,-1), 2, colors.black),\n     ('ALIGN', (1,1), (-1,-1), 'RIGHT')]\n    )\n    ", styleSheet['Code']))
    lst.append(PageBreak())
    t = Table(data, colwidths, rowheights)
    t.setStyle(COLORED_GRID_STYLE)
    lst.append(Paragraph('This is COLORED_GRID_STYLE\n', styleSheet['BodyText']))
    lst.append(t)
    lst.append(Paragraph('\n    It was created as follows:\n    ', styleSheet['BodyText']))
    lst.append(Preformatted("\nCOLORED_GRID_STYLE = TableStyle(\n    [('INNERGRID', (0,0), (-1,-1), 0.25, colors.black),\n     ('BOX', (0,0), (-1,-1), 2, colors.red),\n     ('LINEBELOW', (0,0), (-1,0), 2, colors.black),\n     ('LINEAFTER', (0,0), (0,-1), 2, colors.black),\n     ('ALIGN', (1,1), (-1,-1), 'RIGHT')]\n    )\n    ", styleSheet['Code']))
    t = Table(data, colwidths, rowheights)
    t.setStyle(LIST_STYLE)
    lst.append(Paragraph('This is LIST_STYLE\n', styleSheet['BodyText']))
    lst.append(t)
    lst.append(Paragraph('\n    It was created as follows:\n    ', styleSheet['BodyText']))
    lst.append(Preformatted("\nLIST_STYLE = TableStyle(\n    [('LINEABOVE', (0,0), (-1,0), 2, colors.green),\n     ('LINEABOVE', (0,1), (-1,-1), 0.25, colors.black),\n     ('LINEBELOW', (0,-1), (-1,-1), 2, colors.green),\n     ('ALIGN', (1,1), (-1,-1), 'RIGHT')]\n    )\n    ", styleSheet['Code']))
    t = Table(data, colwidths, rowheights)
    ts = TableStyle([
        ('LINEABOVE', (0, 0), (-1, 0), 2, colors.green),
        ('LINEABOVE', (0, 1), (-1, -1), 0.25, colors.black),
        ('LINEBELOW', (0, -1), (-1, -1), 3, colors.green, 'butt'),
        ('LINEBELOW', (0, -1), (-1, -1), 1, colors.white, 'butt'),
        ('ALIGN', (1, 1), (-1, -1), 'RIGHT'),
        ('TEXTCOLOR', (0, 1), (0, -1), colors.red),
        ('BACKGROUND', (0, 0), (-1, 0), colors.Color(0, 0.69999999999999996, 0.69999999999999996))])
    t.setStyle(ts)
    lst.append(Paragraph('This is a custom style\n', styleSheet['BodyText']))
    lst.append(t)
    lst.append(Paragraph('\n    It was created as follows:\n    ', styleSheet['BodyText']))
    lst.append(Preformatted("\n   ts = TableStyle(\n    [('LINEABOVE', (0,0), (-1,0), 2, colors.green),\n     ('LINEABOVE', (0,1), (-1,-1), 0.25, colors.black),\n     ('LINEBELOW', (0,-1), (-1,-1), 3, colors.green,'butt'),\n     ('LINEBELOW', (0,-1), (-1,-1), 1, colors.white,'butt'),\n     ('ALIGN', (1,1), (-1,-1), 'RIGHT'),\n     ('TEXTCOLOR', (0,1), (0,-1), colors.red),\n     ('BACKGROUND', (0,0), (-1,0), colors.Color(0,0.7,0.7))]\n    )\n    ", styleSheet['Code']))
    data = (('', 'Jan\nCold', 'Feb\n', 'Mar\n', 'Apr\n', 'May\n', 'Jun\nHot', 'Jul\n', 'Aug\nThunder', 'Sep\n', 'Oct\n', 'Nov\n', 'Dec\n'), ('Mugs', 0, 4, 17, 3, 21, 47, 12, 33, 2, -2, 44, 89), ('T-Shirts', 0, 42, 9, -3, 16, 4, 72, 89, 3, 19, 32, 119), ('Key Ring', 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 2, 13), ('Hats', 893, 912, '1,212', 643, 789, 159, 888, '1,298', 832, 453, '1,344', '2,843'))
    c = list(colwidths)
    c[0] = None
    c[8] = None
    t = Table(data, c, [
        None] + list(rowheights[1:]))
    t.setStyle(LIST_STYLE)
    lst.append(Paragraph("\n        This is a LIST_STYLE table with the first rowheight set to None ie automatic.\n        The top row cells are split at a newline '\\n' character. The first and August\n        column widths were also set to None.\n    ", styleSheet['BodyText']))
    lst.append(t)
    lst.append(Paragraph('\n        The red numbers should be aligned LEFT &amp; BOTTOM, the blue RIGHT &amp; TOP\n        and the green CENTER &amp; MIDDLE.\n    ', styleSheet['BodyText']))
    XY = [
        [
            'X00y',
            'X01y',
            'X02y',
            'X03y',
            'X04y'],
        [
            'X10y',
            'X11y',
            'X12y',
            'X13y',
            'X14y'],
        [
            'X20y',
            'X21y',
            'X22y',
            'X23y',
            'X24y'],
        [
            'X30y',
            'X31y',
            'X32y',
            'X33y',
            'X34y']]
    t = Table(XY, 5 * [
        0.59999999999999998 * inch], 4 * [
        0.59999999999999998 * inch])
    t.setStyle([
        ('ALIGN', (1, 1), (-2, -2), 'LEFT'),
        ('TEXTCOLOR', (1, 1), (-2, -2), colors.red),
        ('VALIGN', (0, 0), (1, -1), 'TOP'),
        ('ALIGN', (0, 0), (1, -1), 'RIGHT'),
        ('TEXTCOLOR', (0, 0), (1, -1), colors.blue),
        ('ALIGN', (0, -1), (-1, -1), 'CENTER'),
        ('VALIGN', (0, -1), (-1, -1), 'MIDDLE'),
        ('TEXTCOLOR', (0, -1), (-1, -1), colors.green),
        ('INNERGRID', (0, 0), (-1, -1), 0.25, colors.black),
        ('BOX', (0, 0), (-1, -1), 0.25, colors.black)])
    lst.append(t)
    data = [
        ('alignment', 'align\nalignment'),
        ('bulletColor', 'bulletcolor\nbcolor'),
        ('bulletFontName', 'bfont\nbulletfontname'),
        ('bulletFontSize', 'bfontsize\nbulletfontsize'),
        ('bulletIndent', 'bindent\nbulletindent'),
        ('firstLineIndent', 'findent\nfirstlineindent'),
        ('fontName', 'face\nfontname\nfont'),
        ('fontSize', 'size\nfontsize'),
        ('leading', 'leading'),
        ('leftIndent', 'leftindent\nlindent'),
        ('rightIndent', 'rightindent\nrindent'),
        ('spaceAfter', 'spaceafter\nspacea'),
        ('spaceBefore', 'spacebefore\nspaceb'),
        ('textColor', 'fg\ntextcolor\ncolor')]
    t = Table(data)
    t.setStyle([
        ('VALIGN', (0, 0), (-1, -1), 'TOP'),
        ('INNERGRID', (0, 0), (-1, -1), 0.25, colors.black),
        ('BOX', (0, 0), (-1, -1), 0.25, colors.black)])
    lst.append(t)
    t = Table([
        ('Attribute', 'Synonyms'),
        ('alignment', 'align, alignment'),
        ('bulletColor', 'bulletcolor, bcolor'),
        ('bulletFontName', 'bfont, bulletfontname'),
        ('bulletFontSize', 'bfontsize, bulletfontsize'),
        ('bulletIndent', 'bindent, bulletindent'),
        ('firstLineIndent', 'findent, firstlineindent'),
        ('fontName', 'face, fontname, font'),
        ('fontSize', 'size, fontsize'),
        ('leading', 'leading'),
        ('leftIndent', 'leftindent, lindent'),
        ('rightIndent', 'rightindent, rindent'),
        ('spaceAfter', 'spaceafter, spacea'),
        ('spaceBefore', 'spacebefore, spaceb'),
        ('textColor', 'fg, textcolor, color')])
    t.repeatRows = 1
    t.setStyle([
        ('FONT', (0, 0), (-1, 1), 'Times-Bold', 10, 12),
        ('FONT', (0, 1), (-1, -1), 'Courier', 8, 8),
        ('VALIGN', (0, 0), (-1, -1), 'MIDDLE'),
        ('INNERGRID', (0, 0), (-1, -1), 0.25, colors.black),
        ('BOX', (0, 0), (-1, -1), 0.25, colors.black),
        ('BACKGROUND', (0, 0), (-1, 0), colors.green),
        ('BACKGROUND', (0, 1), (-1, -1), colors.pink),
        ('ALIGN', (0, 0), (-1, 0), 'CENTER'),
        ('ALIGN', (0, 1), (0, -1), 'LEFT'),
        ('ALIGN', (-1, 1), (-1, -1), 'RIGHT'),
        ('FONT', (0, 0), (-1, 0), 'Times-Bold', 12),
        ('ALIGN', (1, 1), (1, -1), 'CENTER')])
    lst.append(t)
    lst.append(Table(XY, style = [
        ('FONT', (0, 0), (-1, -1), 'Times-Roman', 5, 6),
        ('GRID', (0, 0), (-1, -1), 0.25, colors.blue)]))
    lst.append(Table(XY, style = [
        ('FONT', (0, 0), (-1, -1), 'Times-Roman', 10, 12),
        ('GRID', (0, 0), (-1, -1), 0.25, colors.black)]))
    lst.append(Table(XY, style = [
        ('FONT', (0, 0), (-1, -1), 'Times-Roman', 20, 24),
        ('GRID', (0, 0), (-1, -1), 0.25, colors.red)]))
    lst.append(PageBreak())
    data = [
        [
            '00',
            '01',
            '02',
            '03',
            '04'],
        [
            '10',
            '11',
            '12',
            '13',
            '14'],
        [
            '20',
            '21',
            '22',
            '23',
            '24'],
        [
            '30',
            '31',
            '32',
            '33',
            '34']]
    t = Table(data, style = [
        ('GRID', (0, 0), (-1, -1), 0.5, colors.grey),
        ('GRID', (1, 1), (-2, -2), 1, colors.green),
        ('BOX', (0, 0), (1, -1), 2, colors.red),
        ('BOX', (0, 0), (-1, -1), 2, colors.black),
        ('LINEABOVE', (1, 2), (-2, 2), 1, colors.blue),
        ('LINEBEFORE', (2, 1), (2, -2), 1, colors.pink),
        ('BACKGROUND', (0, 0), (0, 1), colors.pink),
        ('BACKGROUND', (1, 1), (1, 2), colors.lavender),
        ('BACKGROUND', (2, 2), (2, 3), colors.orange)])
    lst.append(t)
    lst.append(Spacer(0, 6))
    for s in t.split(4 * inch, 30):
        lst.append(s)
        lst.append(Spacer(0, 6))
    
    lst.append(Spacer(0, 6))
    for s in t.split(4 * inch, 36):
        lst.append(s)
        lst.append(Spacer(0, 6))
    
    lst.append(Spacer(0, 6))
    for s in t.split(4 * inch, 56):
        lst.append(s)
        lst.append(Spacer(0, 6))
    
    import os
    import reportlab.platypus as reportlab
    I = Image(os.path.join(os.path.dirname(reportlab.platypus.__file__), '..', 'tools', 'pythonpoint', 'demos', 'leftlogo.gif'))
    I.drawHeight = 1.25 * inch * I.drawHeight / I.drawWidth
    I.drawWidth = 1.25 * inch
    P = Paragraph('<para align=center spaceb=3>The <b>ReportLab Left <font color=red>Logo</font></b> Image</para>', styleSheet['BodyText'])
    data = [
        [
            'A',
            'B',
            'C',
            Paragraph('<b>A pa<font color=red>r</font>a<i>graph</i></b><super><font color=yellow>1</font></super>', styleSheet['BodyText']),
            'D'],
        [
            '00',
            '01',
            '02',
            [
                I,
                P],
            '04'],
        [
            '10',
            '11',
            '12',
            [
                I,
                P],
            '14'],
        [
            '20',
            '21',
            '22',
            '23',
            '24'],
        [
            '30',
            '31',
            '32',
            '33',
            '34']]
    t = Table(data, style = [
        ('GRID', (1, 1), (-2, -2), 1, colors.green),
        ('BOX', (0, 0), (1, -1), 2, colors.red),
        ('LINEABOVE', (1, 2), (-2, 2), 1, colors.blue),
        ('LINEBEFORE', (2, 1), (2, -2), 1, colors.pink),
        ('BACKGROUND', (0, 0), (0, 1), colors.pink),
        ('BACKGROUND', (1, 1), (1, 2), colors.lavender),
        ('BACKGROUND', (2, 2), (2, 3), colors.orange),
        ('BOX', (0, 0), (-1, -1), 2, colors.black),
        ('GRID', (0, 0), (-1, -1), 0.5, colors.black),
        ('VALIGN', (3, 0), (3, 0), 'BOTTOM'),
        ('BACKGROUND', (3, 0), (3, 0), colors.limegreen),
        ('BACKGROUND', (3, 1), (3, 1), colors.khaki),
        ('ALIGN', (3, 1), (3, 1), 'CENTER'),
        ('BACKGROUND', (3, 2), (3, 2), colors.beige),
        ('ALIGN', (3, 2), (3, 2), 'LEFT')])
    t._argW[3] = 1.5 * inch
    lst.append(t)
    lst.append(PageBreak())
    colWidths = [
        24] * 5
    rowHeight = [
        20] * 5
    data = [
        [
            'A',
            'BBBBB',
            'C',
            'D',
            'E'],
        [
            '00',
            '01',
            '02',
            '03',
            '04'],
        [
            '10',
            '11',
            '12',
            '13',
            '14'],
        [
            '20',
            '21',
            '22',
            '23',
            '24'],
        [
            '30',
            '31',
            '32',
            '33',
            '34']]
    sty = [
        ('ALIGN', (0, 0), (-1, -1), 'CENTER'),
        ('VALIGN', (0, 0), (-1, -1), 'TOP'),
        ('GRID', (0, 0), (-1, -1), 1, colors.green),
        ('BOX', (0, 0), (-1, -1), 2, colors.red),
        ('SPAN', (1, 0), (3, 0)),
        ('BACKGROUND', (1, 0), (1, 0), colors.khaki),
        ('SPAN', (0, 2), (-1, 2)),
        ('SPAN', (0, 0), (0, 1)),
        ('BACKGROUND', (0, 0), (0, 0), colors.cyan),
        ('LINEBELOW', (0, 'splitlast'), (-1, 'splitlast'), 1, colors.white, 'butt')]
    t = Table(data, style = sty, colWidths = [
        20] * 5, rowHeights = [
        20] * 5)
    lst.append(t)
    sty = [
        ('GRID', (0, 0), (-1, -1), 1, colors.green),
        ('BOX', (0, 0), (-1, -1), 2, colors.red)]
    data = [ [
        str(i),
        Paragraph('xx ' * (i % 10), styleSheet['BodyText']),
        Paragraph('blah ' * (i % 40), styleSheet['BodyText'])] for i in xrange(500) ]
    t = LongTable(data, style = sty, colWidths = [
        50,
        100,
        200])
    lst.append(t)
    SimpleDocTemplate('tables.pdf', showBoundary = 1).build(lst)

if __name__ == '__main__':
    test()

